#!/usr/bin/env python

from __future__ import print_function
import nlopt
import numpy as np


def myfunc(x, grad):
    if grad.size > 0:
        grad[0] = 0.0
        grad[1] = 0.5 / np.sqrt(x[1])
    return np.sqrt(x[1])

def myconstraint(x, grad, a, b):
    if grad.size > 0:
        grad[0] = 3 * a * (a*x[0] + b)**2
        grad[1] = -1.0
    return (a*x[0] + b)**3 - x[1]

opt = nlopt.opt(nlopt.LD_MMA, 2)
opt.set_lower_bounds([-float('inf'), 0])
opt.set_min_objective(myfunc)
opt.add_inequality_constraint(lambda x, grad: myconstraint(x,grad, 2, 0), 1e-8)
opt.add_inequality_constraint(lambda x, grad: myconstraint(x,grad, -1, 1), 1e-8)
opt.set_xtol_rel(1e-4)
x0 = [1.234, 5.678]
x = opt.optimize(x0)
minf = opt.last_optimum_value()
print('optimum at ', x)
print('minimum value = ', minf)
print('result code = ', opt.last_optimize_result())
print('nevals = ', opt.get_numevals())
print('initial step =', opt.get_initial_step(x0))



from typing import Dict, List

import os
import sys

from pathlib import Path
from subprocess import check_call

from extension_utils import SYSTEM_WINDOWS, create_directory  # , python_inc, site


def execute_command(
    cmd: List[str], cwd: Path = Path.cwd(), env: Dict[str, str] = os.environ
) -> None:
    print(f"--- {cwd.as_posix()}: {' '.join(cmd)}")
    check_call(cmd, cwd=cwd, env=env)


def build_nlopt(
    *, install_prefix: str = None, build_dir: Path = None, clean=True
) -> Path:

    source_dir = Path(__file__).parent.absolute() / "3rd-party" / "nlopt"
    if not source_dir.exists():
        execute_command(
            ["git", "submodule", "update", "--init", "--recursive", "--depth=1"]
        )

    # TODO dvp: check if something is to be done to support build isolation, use build_ext.build_temp probably?
    if build_dir is None:
        build_dir = create_directory(source_dir / "build", clean=clean)

    if install_prefix is None:
        install_prefix = sys.prefix
        if SYSTEM_WINDOWS:
            install_prefix = os.path.join(install_prefix, "Library")

    cmd = [
        "cmake",
        "-LAH",
        f"-DCMAKE_INSTALL_PREFIX={install_prefix}",
        f"-DPYTHON_EXECUTABLE={sys.executable}",
        "-DNLOPT_GUILE=OFF",
        "-DNLOPT_MATLAB=OFF",
        "-DNLOPT_OCTAVE=OFF",
        source_dir.as_posix(),
    ]

    execute_command(
        cmd=cmd,
        cwd=build_dir,
    )

    execute_command(
        [
            "cmake",
            "--build",
            ".",
            "--config",
            "Release",
            "--",
            "-m" if SYSTEM_WINDOWS else "-j2",
        ],
        cwd=build_dir,
    )

    execute_command(
        [
            "cmake",
            "--install",
            ".",
        ],
        cwd=build_dir,
    )

    return build_dir


if __name__ == "__main__":
    build_nlopt()



from typing import List, Union

import platform
import shutil

from pathlib import Path

SYSTEM_WINDOWS = platform.system() == "Windows"

extra_compile_args = ["/O2"] if SYSTEM_WINDOWS else ["-O3", "-w"]


def create_directory(path: Path, clean: bool = True) -> Path:
    if clean and path.exists():
        shutil.rmtree(path)
    path.mkdir(exist_ok=True, parents=True)
    return path


def check_directories(*directories: str) -> None:
    for directory in directories:
        if not Path(directory).is_dir():
            raise EnvironmentError(f"The directory {directory} does not exist")


def insert_directories(
    destination: List[str], value: Union[str, List[str]]
) -> List[str]:
    dirs = []
    if isinstance(value, list):
        dirs.extend(value)
    elif value not in destination:
        dirs.append(value)
    for old in destination:
        if old not in dirs:
            dirs.append(old)
    return dirs



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['mckit',
 'mckit.cli',
 'mckit.cli.commands',
 'mckit.parser',
 'mckit.parser.common',
 'mckit.utils']

package_data = \
{'': ['*'], 'mckit': ['data/*', 'src/*']}

install_requires = \
['DateTime>=4.3,<5.0',
 'Pygments>=2.7.2,<3.0.0',
 'atomicwrites>=1.4.0,<2.0.0',
 'attrs>=20.2.0,<21.0.0',
 'click-loguru>=1.3.4,<2.0.0',
 'click>=7.1.2,<8.0.0',
 'colorama>=0.4.4,<0.5.0',
 'mkl-devel>=2021.1.1,<2022.0.0',
 'mkl-include>=2021.1.1,<2022.0.0',
 'mkl>=2021.1.1,<2022.0.0',
 'numpy>=1.19.4,<2.0.0',
 'ply>=3.11,<4.0',
 'python-dotenv>=0.15.0,<0.16.0',
 'scipy>=1.5.3,<2.0.0',
 'sly>=0.4,<0.5',
 'tomlkit>=0.7.0,<0.8.0',
 'tqdm>=4.55.1,<5.0.0']

extras_require = \
{':python_version < "3.8"': ['importlib-metadata>=2.0.0,<3.0.0'],
 ':python_version < "3.8" and sys_platform == "Windows"': ['certifi==2020.6.20']}

entry_points = \
{'console_scripts': ['mckit = mckit.cli.runner:mckit']}

setup_kwargs = {
    'name': 'mckit',
    'version': '0.5.10',
    'description': 'Tools to process MCNP models and results',
    'long_description': '.. image:: https://img.shields.io/badge/Maintained%3F-yes-green.svg\n   :target: https://github.com/dvp2015/mckit/graphs/commit-activity\n\n.. image:: https://github.com/dvp2015/mckit/workflows/Tests/badge.svg\n   :target: https://github.com/dvp2015/mckit/actions?workflow=Tests\n\n.. image:: https://codecov.io/gh/dvp2015/mckit/branch/devel/graph/badge.svg?token=05OFBQS3RX\n   :target: https://codecov.io/gh/dvp2015/mckit\n\n.. image:: https://img.shields.io/badge/code%20style-black-000000.svg\n   :target: https://github.com/psf/black\n\n.. image:: https://img.shields.io/badge/%20imports-isort-%231674b1?style=flat&labelColor=ef8336\n    :target: https://pycqa.github.io/isort/\n\n.. image:: https://img.shields.io/github/license/dvp2015/mckit\n   :target: https://github.com/dvp2015/mckit\n\n\n\nMCKIT: MCNP model and results processing utilities\n==================================================\n\nThe mckit package provides a programming framework and command line tools to manipulate complex MCNP models.\nWhen a model is rather complex and its description occupies thousands of text lines it becomes hard to modify it and integrate several model manually.\nThe package automates integration process.\n\n.. TODO The complete documentation is available in the following languages:\n\n.. * `English documentation`_\n.. * `Russian documentation`_\n\n.. .. _English documentation: https://mckit.readthedocs.io/en/latest/\n.. .. _Russian documentation: https://mckit.readthedocs.io/ru/latest/\n\n.. contents:: Table of contents\n\nUsage\n-----\n\nCommand line interface\n~~~~~~~~~~~~~~~~~~~~~~\n\n.. code-block:: bash\n\n    Usage: mckit [OPTIONS] COMMAND [ARGS]...\n\n      Tools to process MCNP models and results\n\n    Options:\n      --override / --no-override\n      --version                   Show the version and exit.\n      -v, --verbose               Log debugging info to stderr.  [default: False]\n      -q, --quiet                 Suppress info to stderr.  [default: False]\n      --logfile / --no-logfile    Log to file.  [default: True]\n      --profile_mem               Profile peak memory use.  [default: False]\n      --help                      Show this message and exit.\n\n    Commands:\n      check      Read MCNP model(s) and show statistics and clashes.\n      compose    Merge universes and envelopes into MCNP model using merge...\n      concat     Concat text files.\n      decompose  Separate an MCNP model to envelopes and filling universes\n      split      Splits MCNP model to text portions (opposite to concat)\n      transform  Transform MCNP model(s) with one of specified transformation.\n\n\nLibrary\n~~~~~~~\n\nThe library allows subtraction and merging models, renaming objects (cells, surfaces, compositions, universes),\nsimplification of cell expressions (removing redundant surfaces), homogenization, computation of cell volumes and\nmaterial masses, and more.\n\n.. code-block:: python\n\n    LOG.info("Loading c-model envelopes")\n    envelopes = load_model(str(CMODEL_ROOT / "c-model.universes/envelopes.i"))\n\n    cells_to_fill = [11, 14, 75]\n    cells_to_fill_indexes = [c - 1 for c in cells_to_fill]\n\n    LOG.info("Attaching bounding boxes to c-model envelopes %s", cells_to_fill)\n    attach_bounding_boxes(\n        [envelopes[i] for i in cells_to_fill_indexes], tolerance=5.0, chunk_size=1\n    )\n    LOG.info("Backing up original envelopes")\n    envelopes_original = envelopes.copy()\n\n    antenna_envelop.rename(start_cell=200000, start_surf=200000)\n\n    LOG.info("Subtracting antenna envelop from c-model envelopes %s", cells_to_fill)\n    envelopes = subtract_model_from_model(\n        envelopes, antenna_envelop, cells_filter=lambda c: c in cells_to_fill\n    )\n    LOG.info("Adding antenna envelop to c-model envelopes")\n    envelopes.add_cells(antenna_envelop, name_rule="clash")\n    envelopes_path = "envelopes+antenna-envelop.i"\n    envelopes.save(envelopes_path)\n    LOG.info("The model of HFSR in envelopes is saved to %s", envelopes_path)\n\n\n\nInstallation\n------------\n\nInstalling from pypi:\n\n.. code-block:: bash\n\n    pip3 install mckit\n\n\nInstalling from github.com:\n\n.. code-block:: bash\n\n    pip3 install git+https://github.com/MC-kit/mckit.git\n\n\nVersioning\n----------\n\nThis software follows `Semantic Versioning`_\n\n.. _Semantic Versioning: http://semver.org/\n\n\nContributors\n------------\n\n* `Roman Rodionov <mailto:r.rodionov@iterrf.ru>`_\n* `Dmitri Portnov <mailto:dmitri_portnov@yahoo.com>`_\n',
    'author': 'rrn',
    'author_email': 'r.rodionov@iterrf.ru',
    'maintainer': 'dpv2015',
    'maintainer_email': 'dmitri_portnov@yahoo.com',
    'url': 'https://github.com/rorni/mckit',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'extras_require': extras_require,
    'entry_points': entry_points,
    'python_requires': '>=3.7,<4.0',
}
from build import *
build(setup_kwargs)

setup(**setup_kwargs)



# noxfile.py
"""
    Nox sessions.

    See `Cjolowicz's article <https://cjolowicz.github.io/posts/hypermodern-python-03-linting>`_
"""
from typing import Any, Generator, List

import os
import platform
import tempfile

from contextlib import contextmanager
from glob import glob
from pathlib import Path

import nox

from nox.sessions import Session

# TODO dvp: uncomment when code and docs are more mature
nox.options.sessions = (
    "safety",
    "isort",
    "black",
    # "lint",
    # "mypy",
    "xdoctest",
    "tests",
    # "codecov",
    # "docs",
)

locations = "mckit", "tests", "noxfile.py", "docs/source/conf.py"

supported_pythons = "3.9 3.8 3.7".split()
black_pythons = "3.9"
mypy_pythons = "3.9"
lint_pythons = "3.9"

on_windows = platform.system() == "Windows"


@contextmanager
def collect_dev_requirements(session: Session) -> Generator[str, None, None]:
    req_path = os.path.join(tempfile.gettempdir(), os.urandom(24).hex())
    try:
        session.run(
            "poetry",
            "export",
            "--dev",
            "--without-hashes",
            "--format=requirements.txt",
            f"--output={req_path}",
            external=True,
        )
        yield req_path
    finally:
        os.unlink(req_path)


# see https://stackoverflow.com/questions/59768651/how-to-use-nox-with-poetry
def install_with_constraints(session: Session, *args: str, **kwargs: Any) -> None:
    """
    Install packages constrained by Poetry's lock file.

    This function is a wrapper for nox.sessions.Session.install. It
    invokes pip to install packages inside of the session's virtualenv.
    Additionally, pip is passed a constraints file generated from
    Poetry's lock file, to ensure that the packages are pinned to the
    versions specified in poetry.lock. This allows you to manage the
    packages as Poetry development dependencies.

    Arguments:
        session: The Session object.
        args: Command-line arguments for pip.
        kwargs: Additional keyword arguments for Session.install.
    """
    with collect_dev_requirements(session) as req_path:
        session.install(f"--constraint={req_path}", *args, **kwargs)


@nox.session(python=supported_pythons, venv_backend="venv")
def tests(session: Session) -> None:
    """Run the test suite."""
    args = session.posargs or ["--cov", "-m", "not e2e"]
    session.run("poetry", "install", "--no-dev", external=True)
    install_with_constraints(session, "pytest", "pytest-cov", "pytest-mock", "coverage")
    path = Path(session.bin).parent
    if on_windows:
        session.bin_paths.insert(
            0, str(path / "Library/bin")
        )  # here all the DLLs should be installed
    session.log(f"Session path: {session.bin_paths}")
    session.run("pytest", env={"LD_LIBRARY_PATH": str(path / "lib")}, *args)
    if "--cov" in args:
        session.run("coverage", "report", "--show-missing", "--skip-covered")
        session.run("coverage", "html")


@nox.session(python=lint_pythons)
def lint(session: Session) -> None:
    """Lint using flake8."""
    args = session.posargs or locations
    install_with_constraints(
        session,
        "flake8",
        "flake8-annotations",
        "flake8-bandit",
        "flake8-black",
        "flake8-bugbear",
        "flake8-docstrings",
        "flake8-import-order",
        "darglint",
    )
    session.run("flake8", *args)


@nox.session(python=black_pythons)  # TODO dvp: this doesn't work with 3.8 so far
def black(session: Session) -> None:
    """Run black code formatter."""
    args = session.posargs or locations
    install_with_constraints(session, "black")
    session.run("black", *args)


@nox.session(python="3.9")
def safety(session: Session) -> None:
    """Scan dependencies for insecure packages."""
    with collect_dev_requirements(session) as req_path:
        install_with_constraints(session, "safety")
        session.run("safety", "check", f"--file={req_path}", "--full-report")


#  This dangerous on ill complex project: may cause cyclic dependency
#  on partial imports ( from ... import).
#  Uncomment when proper imports or noorder directive is applied in sensitive files.
#  Always test after reorganizing ill projects.
#
@nox.session(python="3.9")
def isort(session: Session) -> None:
    """Organize imports"""

    install_with_constraints(session, "isort")
    search_patterns = [
        "*.py",
        "mckit/*.py",
        "tests/*.py",
        "benchmarks/*.py",
        "profiles/*.py",
        #        "adhoc/*.py",
    ]
    files_to_process: List[str] = sum(
        map(lambda p: glob(p, recursive=True), search_patterns), []
    )
    session.run(
        "isort",
        "--check",
        "--diff",
        *files_to_process,
        external=True,
    )


@nox.session(python=mypy_pythons)
def mypy(session: Session) -> None:
    """Type-check using mypy."""
    args = session.posargs or locations
    install_with_constraints(session, "mypy")
    session.run(
        "mypy",
        # "--config",
        # "mypy.ini",  # TODO dvp: compute path to ini-file from test environment: maybe search upward.
        *args,
    )


@nox.session(python=supported_pythons)
def xdoctest(session: Session) -> None:
    """Run examples with xdoctest."""
    args = session.posargs or ["mckit"]
    session.run("poetry", "install", "--no-dev", external=True)
    install_with_constraints(session, "xdoctest")
    session.run("python", "-m", "xdoctest", *args)


@nox.session(python="3.8")
def docs(session: Session) -> None:
    """Build the documentation."""
    session.run("poetry", "install", "--no-dev", external=True)
    install_with_constraints(
        session,
        "sphinx",
        "sphinx-autobuild",
        "numpydoc",
        "sphinxcontrib-htmlhelp",
        "sphinxcontrib-jsmath",
        "sphinxcontrib-napoleon",
        "sphinxcontrib-qthelp",
        "sphinx-autodoc-typehints",
        "sphinx_autorun",
        "sphinx-rtd-theme",
    )
    if session.interactive:
        session.run(
            "sphinx-autobuild",
            "--port=0",
            "--open-browser",
            "docs/source",
            "docs/_build/html",
        )
    else:
        session.run("sphinx-build", "docs/source", "docs/_build")


@nox.session(python="3.8")
def codecov(session: Session) -> None:
    """Upload coverage data."""
    session.run("poetry", "install", "--no-dev", external=True)
    install_with_constraints(
        session,
        "coverage[toml]",
        "pytest",
        "pytest-cov",
        "pytest-mock",
        "coverage",
        "codecov",
    )
    # install_with_constraints(session, "coverage[toml]", "codecov")
    session.run("coverage", "xml", "--fail-under=0")
    session.run("codecov", *session.posargs)


@nox.session(python="3.9", venv_backend="venv")
def test_nox(session: Session) -> None:
    path = Path(session.bin)
    print("bin", path.parent)
    session.run("pip", "install", ".")



import os

from functools import partial

from extension_utils import extra_compile_args
from setuptools import Extension

_libraries = [
    "mkl_rt",
    "nlopt",
]

_sources_root = os.path.join("mckit", "src")
_sources = list(
    map(
        partial(os.path.join, _sources_root),
        ["geometrymodule.c", "box.c", "surface.c", "shape.c", "rbtree.c"],  # noqa
    )
)

geometry_extension = Extension(
    "mckit.geometry",
    sources=_sources,
    libraries=_libraries,
    extra_compile_args=extra_compile_args,
    language="c",
)
